package com.zh.project0512.controller.manage;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.zh.project0512.annotation.AdminTokenValid;
import com.zh.project0512.mapper.AdminMapper;
import com.zh.project0512.mapper.UserEffectRecordsMapper;
import com.zh.project0512.model.Task;
import com.zh.project0512.model.UserEffectRecords;
import com.zh.project0512.model.Video;
import com.zh.project0512.model.dto.RankListDTO;
import com.zh.project0512.service.IAdminService;
import com.zh.project0512.service.IRankService;
import com.zh.project0512.utils.ExcelUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

/**
 * 首页数据统计
 */
@Tag(name = "首页数据统计")
@RestController
@RequestMapping("/manage/statistics")
public class StatisticsController {
    @Autowired
    private IAdminService adminService;
    @Autowired
    private AdminMapper adminMapper;
    @Autowired
    private UserEffectRecordsMapper userEffectRecordsMapper;

    @Operation(summary = "总数据")
    @PostMapping("/data1")
    @AdminTokenValid
    public Result data1() {
        return Result.success(adminMapper.statistics1(), "请求成功");
    }

    @Data
    public static class SSParam {
        @Schema(title = "任务Id")
        private int taskId;
        @Schema(title = "员工Id")
        private int userId;
        @Schema(title = "时间条件", description = "1今日;2本周;3本月;4全年")
        private int dateCondition;
        @Schema(title = "开始时间(yyyy-MM-dd HH:mm:ss)", description = "如果传入dateCondition以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime beginTime;
        @Schema(title = "结束时间(yyyy-MM-dd HH:mm:ss)", description = "如果传入dateCondition以月份区间为条件")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime endTime;
    }

    @Operation(summary = "视频数据")
    @PostMapping("/data2")
    @AdminTokenValid
    public Result data2(@Validated @RequestBody SSParam param) {
        return Result.success(adminService.videoData(param), "请求成功");
    }

    @Data
    static class RankParam {
        @Schema(title = "时间条件", description = "1今日;2本周;3本月;4全年")
        private Integer dateCondition;
        @Schema(title = "1经销商2员工", description = "默认1")
        private Integer type;
        private Integer pageNum;
        private Integer pageSize;
    }

    @Operation(summary = "排行列表")
    @PostMapping("/data3")
    @AdminTokenValid
    public Result data3(@Validated @RequestBody RankParam param) {

        Page pg = MybatisPlusUtil.SetNumPage(param.getPageNum(), param.getPageSize());
        QueryWrapper<UserEffectRecords> qw = new QueryWrapper<>();
        DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDateTime now = LocalDateTime.of(LocalDateTime.now().toLocalDate(), LocalTime.MIN);
        LocalDateTime before = now.with(TemporalAdjusters.firstDayOfMonth());
        if (param.getDateCondition() != null) {
            switch (param.getDateCondition()) {
                case 1:
                    before = now;
                    break;
                case 2:
                    before = now.with(DayOfWeek.MONDAY);
                    break;
                case 3:
                    before = now.with(TemporalAdjusters.firstDayOfMonth());
                    break;
                case 4:
                    before = now.with(TemporalAdjusters.firstDayOfYear());
                    break;
                default:
            }
        }
        qw.between("createAt", before, now.plusDays(1));
        IPage page = (param.getType() != null && param.getType() == 1) ? userEffectRecordsMapper.rankByDepartment(pg, qw) : userEffectRecordsMapper.rankByUser(pg, qw);
        return Result.success(page, "请求成功");
    }

    @Operation(summary = "导出excel", description = "视频数据导出")
    @PostMapping("/excel2")
    @AdminTokenValid
    public void excel(HttpServletResponse response, @Validated @RequestBody SSParam param) throws IOException {
        //这是表头及格式
        String[][] array = {
            {"时间", "播放", "点赞", "收藏", "评论", "转发", "推荐"},
            {"dat", "playNumSum", "commendNumSum", "collectionNumSum", "commentNumSum", "reSendNumSum", "recommendNumSum"},
            {"LocalDateTime", "int", "int", "int", "int", "int", "int"}
        };
        List<Map> l = adminService.videoData(param);

        Workbook wb = ExcelUtil.writeToExcelByList(array, l);
        OutputStream output = response.getOutputStream();
        String fileName = "视频数据.xlsx";
        try {
            fileName = URLEncoder.encode(fileName, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ";" + "filename*=utf-8''" + fileName);
        wb.write(output);
        output.close();
    }

}
